from abi_ds_utils.airflow import write_to_xcom
from abi_ds_utils.aws import get_parameter, get_secret
from abi_ds_utils.spark import getSpark, getGlue

get_spark = getSpark
get_glue = getGlue

__all__ = [
    "get_glue",
    "get_secret",
    "get_spark",
    "get_parameter",
    "write_to_xcom"
]



import json
from typing import Dict
from pathlib import Path


def write_to_xcom(dict_value: Dict) -> Dict:
    """Write `dict_value` for XCOM to pass it to the next task

    ref: https://airflow.apache.org/docs/apache-airflow-providers-cncf-kubernetes/stable/operators.html#how-does-xcom-work

    """
    file_save = Path('/airflow/xcom/return.json')
    file_save.parent.mkdir(parents=True, exist_ok=True)

    # Get data from the file if there is some
    if file_save.is_file():
        with file_save.open('r') as fin:
            dict_restored = json.load(fin)

    else:
        dict_restored = dict()

    # Update dict with new data
    dict_restored.update(dict_value)

    # Overwrite file with updated dict
    with file_save.open('w') as fout:
        json.dump(dict_restored, fout)
    return dict_restored



import json
import base64
from typing import Dict, Optional
import boto3
from botocore.exceptions import ClientError


def get_secret(secret_name: str, region_name: str) -> Dict:
    # Create a Secrets Manager client
    session = boto3.session.Session()
    client = session.client(
        service_name='secretsmanager',
        region_name=region_name
    )

    # In this sample we only handle the specific exceptions for the 'GetSecretValue' API.
    # See https://docs.aws.amazon.com/secretsmanager/latest/apireference/API_GetSecretValue.html
    # We rethrow the exception by default.
    try:
        get_secret_value_response = client.get_secret_value(
            SecretId=secret_name
        )
    except ClientError as e:
        raise e
    else:
        # Decrypts secret using the associated KMS key.
        # Depending on whether the secret is a string or binary, one of these fields will be populated.
        if 'SecretString' in get_secret_value_response:
            secret = get_secret_value_response['SecretString']
        else:
            secret = base64.b64decode(get_secret_value_response['SecretBinary'])
        return json.loads(secret)


def get_parameter(name: str, region: Optional[str] = None) -> Optional[str]:
    """Retrieve secret from Parameter Store.

    :param name: Name of the parameter
    :param region: AWS region otherwise env AWS_DEFAULT_REGION
    :return: Value of parameter
    """
    if region:
        client = boto3.client("ssm", region_name=region)
    else:
        client = boto3.client("ssm")

    try:
        parameter = client.get_parameter(Name=name, WithDecryption=True)
    except ClientError as e:
        raise e
    return parameter.get('Parameter', {}).get('Value')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['abi_ds_utils']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.21.14,<2.0.0', 'pyarrow>=7.0.0', 'pyspark==3.3.2']

setup_kwargs = {
    'name': 'abi-ds-utils',
    'version': '1.2.3',
    'description': 'Utility modules for working with spark, containers, aws and more.',
    'long_description': None,
    'author': 'Martin Matousek',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import os
from pyspark.sql import SparkSession


def getSpark(driver_memory: str = "21g") -> SparkSession:
    spark = (
        SparkSession.builder
        # General
        .master('local[*]')
        .config("spark.driver.maxResultSize", 0)

        # Get 80% of free memory (this might be a bad idea)
        .config("spark.driver.memory", driver_memory)
        .config("spark.dynamicAllocation.enabled", "true")

        # PyArrow for dtypes conversions
        .config("spark.sql.execution.arrow.pyspark.enabled", "true")

        # Jars compatible with the base-notebook image (Python 3.8, PySpark 3.3.2)
        .config('spark.jars.packages', 'org.apache.hadoop:hadoop-aws:3.3.2,io.delta:delta-core_2.12:2.2.0')

        # Delta Lake setup
        .config("spark.hadoop.fs.s3a.connection.maximum", 128)
        .config("spark.sql.extensions", "io.delta.sql.DeltaSparkSessionExtension")
        .config("spark.sql.catalog.spark_catalog", "org.apache.spark.sql.delta.catalog.DeltaCatalog")
        .config("spark.delta.logStore.class", "org.apache.spark.sql.delta.storage.S3SingleDriverLogStore")
    )
    
    spark = spark.config(
        "fs.s3a.aws.credentials.provider",
        "com.amazonaws.auth.DefaultAWSCredentialsProviderChain"
    )
    return spark.getOrCreate()


def getGlue(driver_memory: str = "21g") -> SparkSession:
    spark = (
        SparkSession.builder
        # General
        .config("spark.driver.maxResultSize", 0)

        # Get 80% of free memory (this might be a bad idea)
        .config("spark.driver.memory", driver_memory)
        .config("spark.dynamicAllocation.enabled", "true")

        # PyArrow for dtypes conversions
        .config("spark.sql.execution.arrow.pyspark.enabled", "true")

        # Delta Lake setup
        .config("spark.hadoop.fs.s3a.connection.maximum", 128)
        .config("spark.sql.extensions", "io.delta.sql.DeltaSparkSessionExtension")
        .config("spark.sql.catalog.spark_catalog", "org.apache.spark.sql.delta.catalog.DeltaCatalog")
        .config("spark.delta.logStore.class", "org.apache.spark.sql.delta.storage.S3SingleDriverLogStore")

        # Glue setup
        .config("hive.metastore.client.factory.class", "com.amazonaws.glue.catalog.metastore.AWSGlueDataCatalogHiveClientFactory")
        .config("aws.region", os.environ.get('AWS_DEFAULT_REGION'))
        .config("hive.metastore.glue.catalogid", os.environ.get('GLUE_CATALOG_ID'))
        .enableHiveSupport()
    )

    return spark.getOrCreate()
